require 'mechanize'
require 'certified'

module Atlantis
  module Portal
    class UnsuccessfulAuthenticationError < RuntimeError; end
    class NetworkConnectionError < RuntimeError; end
    class UnexpectedContentError < RuntimeError; end
    class UnknownTeamError < RuntimeError; end

    class Person < Struct.new(:id, :name, :email, :devices, :groups)
      def to_s
        "#{self.id} #{self.name} #{self.email}"
      end
    end

    class Device < Struct.new(:manufacturer, :model, :os_version, :identifier, :name, :platform, :model_name)
      def to_s
        "#{self.identifier} #{self.name}"
      end
    end

    class Group < Struct.new(:id, :alias, :name, :count)
      def to_s
        "#{self.id} #{self.name}"
      end
    end

    class Team < Struct.new(:id, :name, :token)
      def to_s
        "#{self.id} #{self.name} #{self.token}"
      end
    end

    class CommandArguments < Struct.new(:username, :password, :team, :format, :service, :group)
      def to_s
        "#{self.username} #{self.team} #{self.format} #{self.service}"
      end
    end
  end
end

require 'atlantis/portal/agent'
require 'atlantis/portal/service'
require 'atlantis/portal/crashlytics/crashlyticsservice'
require 'atlantis/portal/testflight/testflightservice'