# Monkey Patch Commander::UI to alias password to avoid conflicts
module Commander::UI
  alias :pw :password
end

class String
  include Term::ANSIColor
end

class String
  def is_i?
    !!(self =~ /\A[-+]?[0-9]+\z/)
  end
end

module Atlantis
  module Portal
    module Helpers

      def arguments
        unless @arguments
          @arguments = Atlantis::Portal::CommandArguments.new
        end

        @arguments
      end

      def agent
        unless @agent

          #
          # Web connecting agent
          #

          @agent = Atlantis::Portal::Agent.new

          @agent.instance_eval do
            def username
              @username ||= ask "Username:"
            end

            def password
              @password ||= pw "Password:"
            end
          end
        end

        @agent
      end

      def service

        unless @service
          @service = arguments.service.casecmp('crashlytics') == 0 ? Atlantis::Portal::CrashlyticsService.new : Atlantis::Portal::TestFlightService.new
          @service.arguments = arguments
        end

        @service
      end

      def pluralize(n, singular, plural = nil)
        n.to_i == 1 ? "1 #{singular}" : "#{n} #{plural || singular + 's'}"
      end

      def try
        return unless block_given?

        begin
          yield
        rescue UnsuccessfulAuthenticationError
          say_error "Could not authenticate with the service. Check that your username & password are correct, and that your membership is valid and all pending Terms of Service & agreements are accepted. If this problem continues, try logging into the service from a browser to see what's going on." and abort
        rescue UnknownTeamError
          say_error "Could not find the specified team. Check if your team parameter is correct."
        end
      end

      def output (title, fields, data)

        if arguments.format == "csv"
          output_string = CSV.generate do |csv|
            csv << fields

            data.each do |row|
              csv << array_for_object(fields, row)
            end
          end

        elsif arguments.format == "json"
          objects = []

          data.each do |row|
            object = {}

            fields.each do |field|
              object[field.downcase] = row[field.downcase]
            end

            objects << object
          end

          output_string = JSON.generate(objects)
        else
          output_string = Terminal::Table.new :title => title do |t|
            t << fields
            t.add_separator
            data.each do |row|
              t << array_for_object(fields, row)
            end
          end
        end

        puts output_string
      end
    end

    def array_for_object(fields, row)
      properties = []

      fields.each do |field|
        properties << row[field.downcase]
      end

      return properties
    end
  end
end