require 'mechanize'
require 'security'
require 'uri'
require 'json'
require 'logger'
require 'nokogiri'

include Atlantis::Portal

module Atlantis
  module Portal
    class TestFlightService < ::Service

      def initialize
        super

        self.host = "testflightapp.com"
      end

      def get(uri, parameters = [], referer = nil, headers = {})
        uri = ::File.join("https://#{self.host}", uri) unless /^https?/ === uri

        3.times do
          agent.get(uri, parameters, referer, headers)

          #puts page.body

          return agent.page unless agent.page.respond_to?(:title)

          unless agent.page.parser.at_css('#id_username').nil?
            #puts "Logging in"
            login! and next
          else
            if !@team.nil? && !@team.empty?
              select_team! and next
            else
              return agent.page
            end
          end
        end

        raise UnsuccessfulAuthenticationError
      end

      def list_people(group)

        if group.nil?
          get('/dashboard/team')
        else
          lists = list_groups

          list = lists.find{|p| p.name == group}

          say_warning "No #{group} distribution list found." and abort if list.nil?

          get('/dashboard/team/list/' + list.id)
        end

        people = {}

        agent.page.parser.css("table#member-table tr").each do |row|
          cols = row.css('td')

          #puts cols.length

          if (cols.length > 0)
            person = Person.new

            person.id = cols[0].css("input")[0]['value'];
            person.name = cols[1].text.strip.split.map(&:capitalize).join(' ')
            person.email = cols[2].text.strip
            person.devices = cols[3].text.strip

            people[person.id] = person

          end
        end

        return people
      end

      def list_devices(distribution_list)
        people = list_people(distribution_list)

        people_list = []

        people.each do |id, person|
          people_list << person.id
        end

        agent.post('/dashboard/team/export/devices/', { "members" => people_list.join('|'), "csrfmiddlewaretoken" => agent.page.parser.css("[name='csrfmiddlewaretoken']")[0]['value'] } )

        device_list = agent.page.body.split( /\r?\n/ )

        # Remove first one
        device_list.shift

        devices = []

        device_list.each do |dev|
          #puts dev

          device = Device.new
          device.identifier = dev.split(/\t/)[0]
          device.name = dev.split(/\t/)[1]

          devices << device
        end

        devices
      end

      def list_groups
        get('dashboard/team')

        lists = []

        agent.page.parser.css("nav.vert-nav li a").each do |row|

          unless (row['href'].nil?)
            url = row['href'].split('/')

            #puts url.length

            if (url.length >= 5) && (url[4].is_i?)
              #puts url[3]

              list = Group.new
              list.id = url[4]
              list.name = row.css('> text()').text.strip
              list.count = row.css('span').text.strip

              lists << list
            end

          end
        end

        lists
      end

      def list_teams
        teams = []

        # Selected team

        selected_team = agent.page.parser.css('ul.dropdown-menu li div div.textcontain180 text()')

        team = Team.new
        team.name = selected_team

        teams << team

        all_teams = agent.page.parser.css('ul.ddteamlist li a')
        all_teams.each do |row|
          team = Team.new
          team.id = row['data-team-id']
          team.name = row.text

          teams << team
        end

        teams
      end

      private

      def login!
        if form = agent.page.forms.first
          #puts "LOGGING IN" + self.username

          form.fields_with(type: 'email').first.value = self.username
          form.fields_with(type: 'password').first.value = self.password

          form.submit
        end
      end

      def select_team!
        teams = list_teams

        #puts teams

        # Check if desired team is different to selected team

        if @team != teams[0].name
          #puts "Selecting team..."

          if form = agent.page.form_with(:id => 'team-change')

            team = teams.find{|t| t.name == @team}

            #puts team

            unless (team.nil?)
              form.team = team.id
              form.submit
            end
          end
        end

        @team = ''
      end

    end
  end
end